import pandas as pd
from collections import Counter
import itertools

import pingouin as pg
from scipy.stats import kruskal
from pingouin.power import power_ttest2n

import os
import matplotlib.pyplot as plt
import mne
import matplotlib.pyplot as plt
from mne_connectivity.viz import plot_connectivity_circle

# clean Emotion
em_list = [
    "Etonnement",
    "Joie",
    "Colère",
    "Stupeur",
    "Tristesse",
    "Satiscaction",
    "Emphatie",
    "Intérèt",
    "Peur",
    "Incompréhension",
    "Dégout",
]

em_list_en = [
    "Surprise",
    "Joy",
    "Anger",
    "Stunned",
    "Sadness",
    "Satisfaction",
    "Emphathy",
    "Interest",
    "Fear",
    "Incomprehension",
    "Disgust",
]


def add_emotion(data: pd.DataFrame) -> pd.DataFrame:
    """Add emotion columns to the data using the answer II_5_2

    Args:
        data (pd.Datframe): source data

    Returns:
        data: _description_
    """
    data["emotion_liste"] = data["II_5_2_P1_II_5_2_Propositions_emotions"].apply(
        lambda x: [elt.strip().lower() for elt in x.split(",")]
    )
    for emotion in em_list:
        data[emotion.lower()] = data["emotion_liste"].apply(
            lambda x: 1 if emotion.lower() in x else 0
        )
    return data


def is_association_in_list(association_list: list, test: str) -> int:
    """test if emotion is in list


    Args:
        association_list (list): _description_
        test (str): _description_

    Returns:
        int: _description_
    """
    if test in association_list:
        return 1
    elif (test[1], test[0]) in association_list: # test id opposite association is in list
        return 1
    else:
        return 0


def add_emotion_bigram(data: pd.DataFrame):
    """Add emotion columns to the data using the answer II_5_2

    Args:
        data (pd.Datframe): source data

    Returns:
        data: _description_
    """
    data["emotion_liste"] = data["II_5_2_P1_II_5_2_Propositions_emotions"].apply(
        lambda x: [elt.strip().lower() for elt in x.split(",")]
    )
    data["emotion_association"] = data["emotion_liste"].apply(
        lambda x: list(itertools.combinations(x, r=2))
    )
    association_names = []
    for association in list(
        itertools.combinations([em.lower() for em in em_list], r=2)
    ):
        association_name = "_".join(association).lower()
        data[association_name] = data["emotion_association"].apply(
            lambda x: is_association_in_list(x, association)
        )
        association_names.append(association_name)
    return data, association_names


def build_most_frequent_emotion(emo: pd.DataFrame) -> pd.DataFrame:
    """Compute the emotion distribution over the population in data

    Args:
        emo (pd.DataFrame): data source

    Returns:
        pd.DataFrame:
    """
    count = pd.DataFrame()
    graph_data = pd.DataFrame()
    emo_pers_list = list(
        emo["II_5_2_P1_II_5_2_Propositions_emotions"]
        .apply(lambda x: [elt.strip().lower() for elt in x.split(",")])
        .tolist()
    )
    emo_list = []
    for elt in emo_pers_list:
        for emotion in elt:
            emo_list.append(emotion)
    em_dict = {elt.lower(): 0 for elt in em_list}
    c = {
        key: elt
        for key, elt in dict(Counter(emo_list)).items()
        if key in em_dict.keys()
    }
    c = {**em_dict, **c}
    count["emotion"] = list(c.keys())
    count["freq"] = list(c.values())
    count["score"] = round(count["freq"] / len(emo), 2) * 100

    return count


def build_graph_emotion(emo: pd.DataFrame) -> pd.DataFrame:
    """build the grpah emotion matrice

    Args:
        emo (pd.DataFrame): _description_

    Returns:
        pd.DataFrame: _description_
    """
    graph_data = pd.DataFrame()
    emo_list = list(
        emo["II_5_2_P1_II_5_2_Propositions_emotions"]
        .apply(lambda x: [elt.strip().lower() for elt in x.split(",")])
        .tolist()
    )
    emo_pair = []
    for elt in emo_list:
        emo_pair = emo_pair + list(itertools.combinations(elt, 2))
    c = Counter(emo_pair)
    graph_data["source"] = list(map(lambda x: x[0], list(c.keys())))
    graph_data["target"] = list(map(lambda x: x[1], list(c.keys())))
    graph_data["score"] = list(c.values())
    graph_data["score"] = graph_data["score"] / len(emo)

    return graph_data


def build_coccurence_mat(graph_data: pd.DataFrame, em_list: list) -> pd.DataFrame:
    """build the cocurence matrice associated with the graph matrice

    Args:
        graph_data (pd.DataFrame): _description_
        em_list (list): _description_

    Returns:
        pd.DataFrame: _description_
    """
    ch = pd.DataFrame(columns=em_list, index=em_list)

    for x in em_list:
        for y in em_list:
            try:
                if (
                    len(
                        graph_data[
                            (graph_data["source"] == x.lower())
                            & (graph_data["target"] == y.lower())
                        ]["score"].values
                    )
                    > 0
                ):
                    a = graph_data[
                        (graph_data["source"] == x.lower())
                        & (graph_data["target"] == y.lower())
                    ]["score"].values[0]
                else:
                    a = 0
                if (
                    len(
                        graph_data[
                            (graph_data["source"] == y.lower())
                            & (graph_data["target"] == x.lower())
                        ]["score"].values
                    )
                    > 0
                ):
                    b = graph_data[
                        (graph_data["source"] == y.lower())
                        & (graph_data["target"] == x.lower())
                    ]["score"].values[0]
                else:
                    b = 0
                ch.loc[x, y] = a + b
            except:
                ch.loc[x, y] = 0
    return ch.astype(float)


def compute_rank_sum_emotion(data: pd.DataFrame, x: str) -> dict:
    """rank sum test for emotion col

    Args:
        data (pd.DataFrame): _description_
        x (str): _description_

    Returns:
        dict: _description_
    """
    a = data[data[x] == 1]
    b = data[data[x] == 0]
    dist_a = build_most_frequent_emotion(a).sort_values(by="emotion")["score"].tolist()
    dist_b = build_most_frequent_emotion(b).sort_values(by="emotion")["score"].tolist()

    stats = pg.wilcoxon(dist_a, dist_b)
    stats["cohen"] = pg.compute_effsize(dist_a, dist_b, eftype="cohen")
    stats["power"] = power_ttest2n(nx=len(a), ny=len(b), d=stats["cohen"], alpha=0.05)

    return stats


def compute_kruskal_emotion(data: pd.DataFrame, x: str) -> dict:
    """kruskal test on emotion col

    Args:
        data (pd.DataFrame): _description_
        x (str): _description_

    Returns:
        dict: result of kruskal test
    """

    dists = []
    for cat in data[x].unique():
        a = data[data[x] == cat]
        dist_a = (
            build_most_frequent_emotion(a).sort_values(by="emotion")["score"].tolist()
        )
        dists.append(dist_a)

    stats = kruskal(*dists)

    return stats


def compute_rank_sum_table_emotion(
    data: pd.DataFrame, col_list: list, seuil=0.01
) -> pd.DataFrame:
    """compute the stats for emotional columns

    Args:
        data (pd.DataFrame): _description_
        col_list (list): _description_
        seuil (float, optional): _description_. Defaults to 0.01.

    Returns:
        pd.DataFrame: _description_
    """
    i = 0
    df_result = pd.DataFrame(
        columns=[
            "x",
            "y",
            "W-val",
            "alternative",
            "p-val",
            "RBC",
            "CLES",
            "cohen",
            "power",
        ]
    )
    for elt in col_list:
        try:
            x = "emotional_response"
            y = elt
            result = compute_rank_sum_emotion(data, elt)
            result["x"] = x
            result["y"] = y
            if result["p-val"][0] <= seuil:
                df_result.loc[i] = result.loc["Wilcoxon"]
                i = i + 1
        except Exception as e:
            print(e)
            continue
    return df_result


def plot_emotion_circle(
    data,
    part,
    group="PTSD_complet_ou_incomplet",
    saving_folder="/home/robin/Code_repo/13_novembre_questionnaire_emmotionel/results/plots",
):
    if part == None:
        group_name = "all_population"
        selected_data = data
    elif part == "exposed":
        group_name = "exposed_population"
        selected_data = data[data["exposition"] == 1]
    elif part == "non_exposed":
        group_name = "exposed_population"
        selected_data = data[data["exposition"] == 1]
    elif part == "A1":
        group_name = "direct_exposition"
        selected_data = data[data["critereA"] == "A1"]

    n = len(selected_data[group].unique()) // 2
    if len(selected_data[group].unique()) % 2 > 0:
        n = n + 1
        print(n)

    fig, _ = plt.subplots(
        n, 2, sharex=True, sharey=True, figsize=(20, 10), facecolor="black"
    )
    title = [group + "_" + str(val) for val in selected_data[group].unique()]
    for ii, cat in enumerate(selected_data[group].unique()):
        d = selected_data[selected_data[group] == cat]
        graph_data = build_graph_emotion(d)
        graph_matrix = build_coccurence_mat(graph_data, em_list)
        plot_connectivity_circle(
            graph_matrix.values,
            em_list_en,
            title=title[ii],
            fontsize_colorbar=6,
            padding=10,
            fontsize_names=18,
            fig=fig,
            subplot=(n, 2, ii + 1),
            show=False,
        )

    plt.savefig(os.path.join(saving_folder, f"{part}_{group_name}_emotion_circle.png"))
