from src.analysis.utils_emotions import em_list
from sklearn.model_selection import StratifiedShuffleSplit
from sklearn.metrics import (
    recall_score,
    precision_score,
    confusion_matrix,
    classification_report,
    roc_auc_score,
)
from sklearn.metrics import plot_roc_curve
from sklearn.dummy import DummyClassifier
from imblearn.under_sampling import RandomUnderSampler
from imblearn.over_sampling import SMOTE
from imblearn.pipeline import Pipeline
from imblearn.ensemble import BalancedRandomForestClassifier
from sklearn.linear_model import LogisticRegression, LinearRegression
from sklearn.svm import LinearSVC, SVC
from interpret.glassbox import ExplainableBoostingClassifier, ClassificationTree
import seaborn as sns
from rfpimp import *
from sklearn.ensemble import RandomForestClassifier
import pandas as pd
import os
import matplotlib.pyplot as plt

plt.rcParams["figure.figsize"] = (15, 15)


def train_evaluate_model(
    table: pd.DataFrame,
    y_name: str,
    cat_col=False,
    tree="irf",
    pred=True,
    sample=False,
    seed=1,
):
    """ train and evaluate a model

    Args:
        table (pd.DataFrame): _description_
        y_name (str): _description_
        cat_col (bool, optional): _description_. Defaults to False.
        tree (str, optional): _description_. Defaults to "irf".
        pred (bool, optional): _description_. Defaults to True.
        sample (bool, optional): _description_. Defaults to False.
        seed (int, optional): _description_. Defaults to 1.

    Returns:
        _type_: _description_
    """
    if cat_col == False:
        X = table[[col for col in table.columns if col != y_name]]
    else:
        X = pd.get_dummies(table[cat_col])
    y = table[y_name]

    sss = StratifiedShuffleSplit(n_splits=1, test_size=0.2, random_state=seed)
    for train_index, test_index in sss.split(X, y):
        X_train, y_train = X.loc[train_index], y.loc[train_index]
        X_test, y_test = X.loc[test_index], y.loc[test_index]

    # dealing with unbalanced
    if sample:
        over_smote = SMOTE(
            sampling_strategy={0: int(len(X_train) * 1.3), 1: int(len(X_train) * 1.3)},
            random_state=seed,
        )
        under_random = RandomUnderSampler(
            sampling_strategy={0: int(len(X_train) * 0.8), 1: int(len(X_train) * 0.8)},
            random_state=seed,
        )

        steps = [("over", over_smote), ["under", under_random]]
        pipeline = Pipeline(steps=steps)

        X_train, y_train = pipeline.fit_resample(X_train, y_train)

    if tree == "rf":
        ebm = RandomForestClassifier(
            n_estimators=30,
            max_depth=3,
            max_features="auto",
            class_weight="balanced",
            random_state=seed,
        )
        ebm.fit(X_train, y_train)
        imp = importances(ebm, X_train, y_train)
        imp["name"] = imp.index

    elif tree == "irf":
        ebm = BalancedRandomForestClassifier(
            n_estimators=30, max_depth=5, max_features="auto", random_state=seed
        )
        ebm.fit(X_train, y_train)
        imp = importances(ebm, X_train, y_train)
        imp["name"] = imp.index

    elif tree == "svc":
        ebm = SVC(C=1, kernel="rbf", class_weight="balanced")
        ebm.fit(X_train, y_train)
        imp = pd.DataFrame()
        imp["name"] = list(X_train.columns)
        imp["importance"] = 0

    elif tree == "logr":
        ebm = LogisticRegression(C=1, penalty="l2", class_weight="balanced")
        ebm.fit(X_train, y_train)
        imp = pd.DataFrame()
        imp["name"] = list(X_train.columns)
        imp["importance"] = ebm.coef_[0]

    elif tree == "ct":
        ebm = ClassificationTree(feature_names=X.columns, random_state=seed)
        ebm.fit(X_train, y_train)

        imp = pd.DataFrame()
        imp["name"] = ebm.feature_names
        imp["importance"] = 0
        imp = imp.sort_values("importance", ascending=False)
    elif tree == "dummy":
        ebm = DummyClassifier(strategy="most_frequent").fit(X_train, y_train)  # prior
        imp = pd.DataFrame(columns=["name", "importance"])

    else:
        ebm = ExplainableBoostingClassifier(
            feature_names=X.columns, random_state=seed, max_leaves=5
        )
        ebm.fit(X_train, y_train.tolist())

        imp = pd.DataFrame()
        imp["name"] = ebm.feature_names
        imp["importance"] = ebm.feature_importances_
        imp = imp.sort_values("importance", ascending=False)

    y_pred = ebm.predict(X_test)
    if pred == True:
        table[y_name + "_pred"] = ebm.predict(X)

    tn, fp, fn, tp = confusion_matrix(y_test, y_pred).ravel()
    specificity = tn / (tn + fp)
    r = {
        "ebm": ebm,
        "critere": y_name,
        "feature": list(X.columns),
        "col": cat_col,
        "precision": precision_score(y_test, y_pred),
        "recall": recall_score(y_test, y_pred),
        "auc_score": roc_auc_score(y_test, y_pred),
        "specificity": specificity,
        "imp": imp,
        "report": classification_report(y_test, y_pred),
    }

    return r, X_test, y_test


def select_data(data: pd.DataFrame, group: str):
    if group == None:
        group_name = "all_population"
        selected_data = data
    elif group == "exposed":
        group_name = "exposed_population"
        selected_data = data[data["exposition"] == 1]
    elif group == "non_exposed":
        group_name = "non_exposed_population"
        selected_data = data[data["exposition"] == 0]
    elif group == "A1":
        group_name = "direct_exposition"
        selected_data = data[data["critereA"] == "A1"]
    elif group == "A3":
        group_name = "deuil_exposition"
        selected_data = data[data["critereA"] == "A3"]
    elif group == "A4":
        group_name = "professional_exposition"
        selected_data = data[data["critereA"] == "A4"]
    elif group == "not_pro":
        group_name = "not_professional_exposition"
        selected_data = data[data["critereA"] != "A4"]
    else:
        return data, "all_population"

    return selected_data, group_name


def main_ml(
    data: pd.DataFrame,
    group: str,
    saving_folder="/home/robin/Code_repo/13_novembre_questionnaire_emmotionel/results",
):
    y_name = "PTSD_complet_ou_incomplet"
    features = (
        [
            "tristesse_peur",
            "joie_tristesse",
            "peur_incompréhension",
            "joie_satiscaction",
            "stupeur_peur",
            "satiscaction_peur",
            "colère_dégout",
            "emphatie_dégout",
        ]
        + [em.strip().lower() for em in em_list]
        + [
            "souvenir_avant_nov",
            "souvenir_apres_nov",
            "souvenir_heure_nov",
            "souvenir_nov",
            "souvenir_crash_avion",
            "souvenir_tgv",
        ]
        + ["norm_h_13_14", "h_perso_enc", "num_h_mois_enc"]
    )

    selected_col = features + [y_name]
    print(f"The selected group is:  {group}")
    cut_data, name = select_data(data, group)

    X = cut_data[selected_col].reset_index(drop=True)
    print(f"Training model using this features: {features}")
    print("training started...")
    r_dummy, X_test, y_test = train_evaluate_model(
        X, y_name, cat_col=features, tree="dummy", sample=False
    )
    r_logr, X_test, y_test = train_evaluate_model(
        X, y_name, cat_col=features, tree="logr", sample=True
    )
    r_rf, X_test, y_test = train_evaluate_model(
        X, y_name, cat_col=features, tree="rf", sample=True
    )
    r_irf, X_test, y_test = train_evaluate_model(
        X, y_name, cat_col=features, tree="irf", sample=False
    )
    r_ebm, X_test, y_test = train_evaluate_model(
        X, y_name, cat_col=features, tree="ebm", sample=False
    )

    print("ploting the ROC-AUC curves")

    disp = plot_roc_curve(r_dummy["ebm"], X_test, y_test)
    plot_roc_curve(r_rf["ebm"], X_test, y_test, ax=disp.ax_)
    plot_roc_curve(r_irf["ebm"], X_test, y_test, ax=disp.ax_)
    plot_roc_curve(r_ebm["ebm"], X_test, y_test, ax=disp.ax_)
    plot_roc_curve(r_logr["ebm"], X_test, y_test, ax=disp.ax_)
    plt.legend(fontsize=13)
    fig_save_name = os.path.join(saving_folder, "plots", f"{group}_roc_auc.png")
    plt.savefig(fig_save_name)

    print("Saving the interpretations...")
    for name, model in {"ebm": r_ebm, "irf": r_irf, "rf": r_rf}.items():
        saving_path = os.path.join(
            saving_folder, "stats", f"machine_learning_{group}_{name}.csv"
        )
        df_imp = model["imp"].reset_index()[:20]
        df_imp.columns = df_imp.columns.str.lower()
        df_imp["features_importances"] = df_imp["importance"].apply(float)
        df_imp["features_names"] = df_imp["name"].apply(str)
        df_imp.to_csv(saving_path)

    print("done !")
