import os
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

from src.analysis.utils_analysis import fac_h_perso


import prince
from adjustText import adjust_text


def encode_media_mca(x):
    if x <= 0:
        return "none"
    elif x <= 1:
        return "low"
    elif x <= 2:
        return "medium"
    else:
        return "high"


def encode_souvenir_mca(x):
    if x <= 1:
        return "weak"
    elif x == 2:
        return "normal"
    else:
        return "precise"


def encode_situation_mattrimoniale_mca(x):
    if x in ["EN COUPLE", "MARIE(E)/PACSE(E)"]:
        return "in_relationship"
    else:
        return "single"


def encode_mode_residence_mca(x):
    if x in ["SEUL"]:
        return "alone"
    else:
        return "not_alone"


def prepare_data_mca(data: pd.DataFrame):
    """Prepara the data for MCA visualisation, (mainly english translation)

    Args:
        data (pd.DataFrame): _description_

    Returns:
        _type_: _description_
    """
    mca_data = data.copy()
    mca_data["situation_matrimoniale"] = mca_data["situation_matrimoniale"].apply(
        encode_situation_mattrimoniale_mca
    )
    mca_data["mode_residence"] = mca_data["mode_residence"].apply(
        encode_mode_residence_mca
    )
    mca_data["media_day"] = mca_data["norm_h_13_14_enc"].apply(encode_media_mca)
    mca_data["media_month"] = mca_data["num_h_mois_enc"].apply(encode_media_mca)
    mca_data["souvenir_hour"] = mca_data["souvenir_heure_nov"].apply(
        encode_souvenir_mca
    )
    mca_data["h_perso"] = data[fac_h_perso[1]].apply(
        lambda x: "yes" if "Oui" in x else "no"
    )
    mca_data["h_family"] = data[fac_h_perso[0]].apply(
        lambda x: "yes" if "Oui" in x else "no"
    )
    mca_data["full_PTSD"] = data["PTSD_probable"].apply(
        lambda x: "yes" if x == 1 else "no"
    )
    mca_data["full_or_partial_PTSD"] = data["PTSD_complet_ou_incomplet"].apply(
        lambda x: "yes" if x == 1 else "no"
    )

    mca_col = [
        "sexe",
        "critereA",
        "situation_matrimoniale",
        "mode_residence",
        "media_day",
        "media_month",
        "souvenir_hour",
        "h_perso",
        "full_PTSD",
        "full_or_partial_PTSD",
        "h_family",
    ]
    return mca_data, mca_col


def compute_mca_inertia(mca, show=False):
    """compute the mca inertia

    Args:
        mca (prince.mca): Multiple correspondance analysis
        show (bool, optional):should we print the results. Defaults to False.
    """
    n_components = len(mca.explained_inertia_)
    explained_variance = mca.explained_inertia_
    cum_explained_variance = np.cumsum(explained_variance)
    idx = np.arange(n_components) + 1
    df_explained_variance = pd.DataFrame(
        [explained_variance, cum_explained_variance],
        index=["explained inertia", "cumulative"],
        columns=idx,
    ).T

    mean_explained_variance = df_explained_variance.iloc[
        :, 0
    ].mean()  # calculate mean explained variance# (iii.) Print explained variance as plain text

    if show == True:
        print("PCA Overview")
        print("=" * 40)
        print("Total: {} components".format(n_components))
        print("-" * 40)
        print("Mean explained variance:", round(mean_explained_variance, 3))
        print("-" * 40)
        print(df_explained_variance.head(20))
        print("-" * 40)

    return df_explained_variance, idx


def plot_inertia(
    df_explained_variance,
    idx,
    mca_name="test",
    limit_df=10,
    saving_folder="/home/robin/Code_repo/13_novembre_questionnaire_emmotionel/results/plots",
    show=False,
):

    mean_explained_variance = df_explained_variance.iloc[:, 0].mean()
    df_explained_variance_limited = df_explained_variance.iloc[
        :limit_df, :
    ]  # make scree plot

    fig, ax1 = plt.subplots(figsize=(15, 6))
    ax1.set_title(
        "Explained inertia across Multiple Correspondence Analysis", fontsize=14
    )
    ax1.set_xlabel("Components", fontsize=12)
    ax1.set_ylabel("Explained Inertia", fontsize=12)

    ax2 = sns.barplot(
        x=idx[:limit_df],
        y="explained inertia",
        data=df_explained_variance_limited,
        palette="summer",
    )

    ax2 = ax1.twinx()
    ax2.grid(False)
    ax2.set_ylabel("Cumulative", fontsize=14)
    ax2 = sns.lineplot(
        x=idx[:limit_df] - 1,
        y="cumulative",
        data=df_explained_variance_limited,
        color="#fc8d59",
    )

    ax1.axhline(mean_explained_variance, ls="--", color="#fc8d59")  # plot mean
    ax1.text(
        -0.8,
        mean_explained_variance + (mean_explained_variance * 0.05),
        "average",
        color="#fc8d59",
        fontsize=14,
    )  # label y axis

    max_y1 = max(df_explained_variance_limited.iloc[:, 0])
    max_y2 = max(df_explained_variance_limited.iloc[:, 1])
    ax1.set(ylim=(0, max_y1 + max_y1 * 0.1))
    ax2.set(ylim=(0, max_y2 + max_y2 * 0.1))

    plt.savefig(os.path.join(saving_folder, f"MCA_INERTIA_{mca_name}.png"))
    if show:
        plt.show()


def compute_mca(
    mca_data,
    mca_col,
    mca_data_name="",
    saving_folder="/home/robin/Code_repo/13_novembre_questionnaire_emmotionel/results/plots",
):

    X = mca_data[mca_col]
    mca = prince.MCA(
        n_components=2,
        n_iter=5,
        copy=True,
        check_input=True,
        engine="auto",
        random_state=42,
    )
    mca = mca.fit(X)

    plt.rcParams["axes.labelsize"] = 20
    plt.rcParams["font.size"] = 15

    ax = mca.plot_coordinates(
        X=X,
        ax=None,
        figsize=(13, 13),
        show_row_points=False,
        row_points_size=10,
        show_row_labels=False,
        show_column_points=True,
        column_points_size=200,
        show_column_labels=True,
        legend_n_cols=1,
    )

    adjust_text(
        list(ax.texts),
        only_move={"points": "y", "texts": "y"},
        arrowprops=dict(arrowstyle="->", color="r", lw=0.5),
    )

    ax.legend(loc="upper left")

    plt.savefig(os.path.join(saving_folder, f"MCA_{mca_data_name}_plot.png"))
    return mca
